<?php

/**
 * moziloCMS Plugin: ContentScroll
 *
 * Makes content that is wider than containter width scrollable.
 *
 * PHP version 5
 *
 * @category PHP
 * @package  PHP_MoziloPlugins
 * @author   DEVMOUNT <mail@devmount.de>
 * @license  GPL
 * @version  GIT: v0.1.2014-06-18
 * @link     https://github.com/devmount-mozilo/ContentScroll/wiki/Dokumentation
 * @see      And the peace of God, which transcends all understanding, will guard
 *           your hearts and your minds in Christ Jesus.
 *           – The Bible
 *
 * Plugin created by DEVMOUNT
 * www.devmount.de
 *
 */

// only allow moziloCMS environment
if (!defined('IS_CMS')) {
    die();
}

/**
 * ContentScroll Class
 *
 * @category PHP
 * @package  PHP_MoziloPlugins
 * @author   DEVMOUNT <mail@devmount.de>
 * @license  GPL
 * @link     https://github.com/devmount-mozilo/ContentScroll
 */
class ContentScroll extends Plugin
{
    // language
    private $_admin_lang;
    private $_cms_lang;

    // plugin information
    const PLUGIN_AUTHOR  = 'DEVMOUNT';
    const PLUGIN_TITLE   = 'ContentScroll';
    const PLUGIN_VERSION = 'v0.1.2014-06-18';
    const MOZILO_VERSION = '2.0';
    const PLUGIN_DOCU
        = 'https://github.com/devmount-mozilo/ContentScroll/wiki/Dokumentation';

    private $_plugin_tags = array(
        'tag1' => '{ContentScroll|<height>|<content>}',
    );

    const LOGO_URL = '../plugins/ContentScroll/logo_pluginconf.png';

    /**
     * set configuration elements, their default values and their configuration
     * parameters
     *
     * @var array $_confdefault
     *      text     => default, type, maxlength, size, regex
     *      textarea => default, type, cols, rows, regex
     *      password => default, type, maxlength, size, regex, saveasmd5
     *      check    => default, type
     *      radio    => default, type, descriptions
     *      select   => default, type, descriptions, multiselect
     */
    private $_confdefault = array(
        'hotSpotScrolling' => array(
            true,
            'check',
        ),
        'hotSpotMouseDownSpeedBooster' => array(
            '3',
            'select',
            array('1', '2', '3', '4', '5', '6', '7', '8', '9', '10'),
            false,
        ),
        'mousewheelScrolling' => array(
            'allDirections',
            'select',
            array('vertical', 'horizontal', 'allDirections'),
            false,
        ),
        'touchScrolling' => array(
            true,
            'check',
        ),
        'manualContinuousScrolling' => array(
            true,
            'check',
        ),
        'autoScrollingMode' => array(
            '',
            'select',
            array('','onStart','auto'),
            false,
        ),
        'autoScrollingDirection' => array(
            'backAndForth',
            'select',
            array(
                'right',
                'left',
                'backAndForth',
                'endlessLoopRight',
                'endlessLoopLeft'
            ),
            false,
        ),
    );

    /**
     * creates plugin content
     *
     * @param string $value Parameter divided by '|'
     *
     * @return string HTML output
     */
    function getContent($value)
    {
        global $CMS_CONF;
        global $syntax;

        // initialize cms lang
        $this->_cms_lang = new Language(
            $this->PLUGIN_SELF_DIR
            . 'lang/cms_language_'
            . $CMS_CONF->get('cmslanguage')
            . '.txt'
        );

        // get language labels
        $label = $this->_cms_lang->getLanguageValue('label');

        // get params
        list($param_height, $param_content)
            = $this->makeUserParaArray($value, false, '|');

        // check params
        if ($param_height == '') {
            $param_height = 100;
        }
        if ($param_content == '') {
            return $this->throwMessage(
                $this->_cms_lang->getLanguageValue('error_nocontent'),
                'ERROR'
            );
        }

        // get conf and set default
        $conf = array();
        foreach ($this->_confdefault as $elem => $default) {
            $conf[$elem] = ($this->settings->get($elem) == '')
                ? $default[0]
                : $this->settings->get($elem);
        }

        // include jquery
        $syntax->insert_jquery_in_head('jquery');

        // include smoothDivScroll CSS
        $syntax->insert_in_head(
            '<!-- the CSS for Smooth Div Scroll -->
             <link rel="Stylesheet" type="text/css"
                href="' . $this->PLUGIN_SELF_URL
                . 'smoothDivScroll/css/smoothDivScroll.css" />'
        );

        // initialize return content, begin plugin content
        $content = '<!-- BEGIN ' . self::PLUGIN_TITLE . ' plugin content --> ';

        // add container
        $content .=
            '<div id="contentscroll" style="height: ' . $param_height . 'px;">'
                . $param_content
            . '</div>';

        // jQuery UI (Custom Download containing only Widget and Effects Core)
        // http://jqueryui.com/download
        $content .=
            '<script type="text/javascript"
                src="' . $this->PLUGIN_SELF_URL
                . 'smoothDivScroll/js/jquery-ui-1.10.3.custom.min.js"
                type="text/javascript">
            </script>'
        ;

        // Latest version (3.1.4) of jQuery Mouse Wheel by Brandon Aaron
        // https://github.com/brandonaaron/jquery-mousewheel
        $content .=
            '<script type="text/javascript"
                src="' . $this->PLUGIN_SELF_URL
                . 'smoothDivScroll/js/jquery.mousewheel.min.js"
                type="text/javascript">
            </script>'
        ;

        // jQuery Kinectic (1.8.2) used for touch scrolling
        // https://github.com/davetayls/jquery.kinetic/
        $content .=
            '<script type="text/javascript"
                src="' . $this->PLUGIN_SELF_URL
                . 'smoothDivScroll/js/jquery.kinetic.min.js"
                type="text/javascript">
            </script>'
        ;

        // Smooth Div Scroll 1.3 minified
        $content .=
            '<script type="text/javascript"
                src="' . $this->PLUGIN_SELF_URL
                . 'smoothDivScroll/js/jquery.smoothdivscroll-1.3-min.js"
                type="text/javascript">
            </script>'
        ;

        // Smooth Div Scroll initialization
        $content .=
            '<script type="text/javascript">
                $(document).ready(function () {
                    $("div#contentscroll").smoothDivScroll({';
        // set configuration
        foreach ($conf as $key => $value) {
            $defaultconf = $this->_confdefault;
            // quote selected values
            if ($defaultconf[$key][1] == 'select') {
                $content .= $key . ': "' . $value . '",';
            } else {
                $content .= $key . ': ' . $value . ',';
            }
        }
        // remove last comma
        $content = substr($content, 0, -1);
        $content .= '});
                });
            </script>'
        ;

        // end plugin content
        $content .= '<!-- END ' . self::PLUGIN_TITLE . ' plugin content --> ';

        return $content;
    }

    /**
     * sets backend configuration elements and template
     *
     * @return Array configuration
     */
    function getConfig()
    {
        $config = array();

        // read configuration values
        foreach ($this->_confdefault as $key => $value) {
            // handle each form type
            switch ($value[1]) {
            case 'text':
                $config[$key] = $this->confText(
                    $this->_admin_lang->getLanguageValue('config_' . $key),
                    $value[2],
                    $value[3],
                    $value[4],
                    $this->_admin_lang->getLanguageValue(
                        'config_' . $key . '_error'
                    )
                );
                break;

            case 'textarea':
                $config[$key] = $this->confTextarea(
                    $this->_admin_lang->getLanguageValue('config_' . $key),
                    $value[2],
                    $value[3],
                    $value[4],
                    $this->_admin_lang->getLanguageValue(
                        'config_' . $key . '_error'
                    )
                );
                break;

            case 'password':
                $config[$key] = $this->confPassword(
                    $this->_admin_lang->getLanguageValue('config_' . $key),
                    $value[2],
                    $value[3],
                    $value[4],
                    $this->_admin_lang->getLanguageValue(
                        'config_' . $key . '_error'
                    ),
                    $value[5]
                );
                break;

            case 'check':
                $config[$key] = $this->confCheck(
                    $this->_admin_lang->getLanguageValue('config_' . $key)
                );
                break;

            case 'radio':
                $descriptions = array();
                foreach ($value[2] as $label) {
                    $descriptions[$label] = $this->_admin_lang->getLanguageValue(
                        'config_' . $key . '_' . $label
                    );
                }
                $config[$key] = $this->confRadio(
                    $this->_admin_lang->getLanguageValue('config_' . $key),
                    $descriptions
                );
                break;

            case 'select':
                $descriptions = array();
                foreach ($value[2] as $label) {
                    $descriptions[$label] = $this->_admin_lang->getLanguageValue(
                        'config_' . $key . '_' . $label
                    );
                }
                $config[$key] = $this->confSelect(
                    $this->_admin_lang->getLanguageValue('config_' . $key),
                    $descriptions,
                    $value[3]
                );
                break;

            default:
                break;
            }
        }

        // read admin.css
        $admin_css = '';
        $lines = file('../plugins/' . self::PLUGIN_TITLE. '/admin.css');
        foreach ($lines as $line_num => $line) {
            $admin_css .= trim($line);
        }

        // add template CSS
        $template = '<style>' . $admin_css . '</style>';

        // build Template
        $template .= '
            <div class="contentscroll-admin-header">
            <span>'
                . $this->_admin_lang->getLanguageValue(
                    'admin_header',
                    self::PLUGIN_TITLE
                )
            . '</span>
            <a href="' . self::PLUGIN_DOCU . '" target="_blank">
            <img style="float:right;" src="' . self::LOGO_URL . '" />
            </a>
            </div>
        </li>
        <li class="mo-in-ul-li ui-widget-content contentscroll-admin-li">
            <div class="contentscroll-admin-subheader">'
            . $this->_admin_lang->getLanguageValue('admin_navigation')
            . '</div>
            <div class="contentscroll-single-conf">
                {hotSpotScrolling_checkbox}
                {hotSpotScrolling_description}
                <span class="contentscroll-admin-default">
                    [' . $this->_confdefault['hotSpotScrolling'][0] .']
                </span>
            </div>
            <div class="contentscroll-single-conf">
                <div class="contentscroll-col-4">
                    {hotSpotMouseDownSpeedBooster_select}
                </div>
                {hotSpotMouseDownSpeedBooster_description}
                <span class="contentscroll-admin-default">
                    [' . $this->_confdefault['hotSpotMouseDownSpeedBooster'][0] .']
                </span>
            </div>
        </li>
        <li class="mo-in-ul-li ui-widget-content contentscroll-admin-li">
            <div class="contentscroll-admin-subheader">'
            . $this->_admin_lang->getLanguageValue('admin_loop')
            . '</div>
            <div class="contentscroll-single-conf">
                {manualContinuousScrolling_checkbox}
                {manualContinuousScrolling_description}
                <span class="contentscroll-admin-default">
                    [' . $this->_confdefault['manualContinuousScrolling'][0] .']
                </span>
            </div>
        </li>
        <li class="mo-in-ul-li ui-widget-content contentscroll-admin-li">
            <div class="contentscroll-admin-subheader">'
            . $this->_admin_lang->getLanguageValue('admin_scrolling')
            . '</div>
            <div class="contentscroll-single-conf">
                {touchScrolling_checkbox}
                {touchScrolling_description}
                <span class="contentscroll-admin-default">
                    [' . $this->_confdefault['touchScrolling'][0] .']
                </span>
            </div>
            <div class="contentscroll-single-conf">
                <div class="contentscroll-col-4">
                    {mousewheelScrolling_select}
                </div>
                {mousewheelScrolling_description}
                <span class="contentscroll-admin-default">
                    [' . $this->_confdefault['mousewheelScrolling'][0] .']
                </span>
            </div>
            <div class="contentscroll-single-conf">
                <div class="contentscroll-col-4">
                    {autoScrollingMode_select}
                </div>
                {autoScrollingMode_description}
                <span class="contentscroll-admin-default">
                    [' . $this->_confdefault['autoScrollingMode'][0] .']
                </span>
            </div>
            <div class="contentscroll-single-conf">
                <div class="contentscroll-col-4">
                    {autoScrollingDirection_select}
                </div>
                {autoScrollingDirection_description}
                <span class="contentscroll-admin-default">
                    [' . $this->_confdefault['autoScrollingDirection'][0] .']
                </span>
        ';

        $config['--template~~'] = $template;

        return $config;
    }

    /**
     * sets default backend configuration elements, if no plugin.conf.php is
     * created yet
     *
     * @return Array configuration
     */
    function getDefaultSettings()
    {
        $config = array('active' => 'true');
        foreach ($this->_confdefault as $elem => $default) {
            $config[$elem] = $default[0];
        }
        return $config;
    }

    /**
     * sets backend plugin information
     *
     * @return Array information
     */
    function getInfo()
    {
        global $ADMIN_CONF;

        $this->_admin_lang = new Language(
            $this->PLUGIN_SELF_DIR
            . 'lang/admin_language_'
            . $ADMIN_CONF->get('language')
            . '.txt'
        );

        // build plugin tags
        $tags = array();
        foreach ($this->_plugin_tags as $key => $tag) {
            $tags[$tag] = $this->_admin_lang->getLanguageValue('tag_' . $key);
        }

        $info = array(
            '<b>' . self::PLUGIN_TITLE . '</b> ' . self::PLUGIN_VERSION,
            self::MOZILO_VERSION,
            $this->_admin_lang->getLanguageValue(
                'description',
                htmlspecialchars($this->_plugin_tags['tag1'], ENT_COMPAT, 'UTF-8')
            ),
            self::PLUGIN_AUTHOR,
            array(
                self::PLUGIN_DOCU,
                self::PLUGIN_TITLE . ' '
                . $this->_admin_lang->getLanguageValue('on_devmount')
            ),
            $tags
        );

        return $info;
    }

    /**
     * creates configuration for text fields
     *
     * @param string $description Label
     * @param string $maxlength   Maximum number of characters
     * @param string $size        Size
     * @param string $regex       Regular expression for allowed input
     * @param string $regex_error Wrong input error message
     *
     * @return Array  Configuration
     */
    protected function confText(
        $description,
        $maxlength = '',
        $size = '',
        $regex = '',
        $regex_error = ''
    ) {
        // required properties
        $conftext = array(
            'type' => 'text',
            'description' => $description,
        );
        // optional properties
        if ($maxlength != '') {
            $conftext['maxlength'] = $maxlength;
        }
        if ($size != '') {
            $conftext['size'] = $size;
        }
        if ($regex != '') {
            $conftext['regex'] = $regex;
        }
        if ($regex_error != '') {
            $conftext['regex_error'] = $regex_error;
        }
        return $conftext;
    }

    /**
     * creates configuration for textareas
     *
     * @param string $description Label
     * @param string $cols        Number of columns
     * @param string $rows        Number of rows
     * @param string $regex       Regular expression for allowed input
     * @param string $regex_error Wrong input error message
     *
     * @return Array  Configuration
     */
    protected function confTextarea(
        $description,
        $cols = '',
        $rows = '',
        $regex = '',
        $regex_error = ''
    ) {
        // required properties
        $conftext = array(
            'type' => 'textarea',
            'description' => $description,
        );
        // optional properties
        if ($cols != '') {
            $conftext['cols'] = $cols;
        }
        if ($rows != '') {
            $conftext['rows'] = $rows;
        }
        if ($regex != '') {
            $conftext['regex'] = $regex;
        }
        if ($regex_error != '') {
            $conftext['regex_error'] = $regex_error;
        }
        return $conftext;
    }

    /**
     * creates configuration for password fields
     *
     * @param string  $description Label
     * @param string  $maxlength   Maximum number of characters
     * @param string  $size        Size
     * @param string  $regex       Regular expression for allowed input
     * @param string  $regex_error Wrong input error message
     * @param boolean $saveasmd5   Safe password as md5 (recommended!)
     *
     * @return Array   Configuration
     */
    protected function confPassword(
        $description,
        $maxlength = '',
        $size = '',
        $regex = '',
        $regex_error = '',
        $saveasmd5 = true
    ) {
        // required properties
        $conftext = array(
            'type' => 'text',
            'description' => $description,
        );
        // optional properties
        if ($maxlength != '') {
            $conftext['maxlength'] = $maxlength;
        }
        if ($size != '') {
            $conftext['size'] = $size;
        }
        if ($regex != '') {
            $conftext['regex'] = $regex;
        }
        $conftext['saveasmd5'] = $saveasmd5;
        return $conftext;
    }

    /**
     * creates configuration for checkboxes
     *
     * @param string $description Label
     *
     * @return Array  Configuration
     */
    protected function confCheck($description)
    {
        // required properties
        return array(
            'type' => 'checkbox',
            'description' => $description,
        );
    }

    /**
     * creates configuration for radio buttons
     *
     * @param string $description  Label
     * @param string $descriptions Array Single item labels
     *
     * @return Array Configuration
     */
    protected function confRadio($description, $descriptions)
    {
        // required properties
        return array(
            'type' => 'select',
            'description' => $description,
            'descriptions' => $descriptions,
        );
    }

    /**
     * creates configuration for select fields
     *
     * @param string  $description  Label
     * @param string  $descriptions Array Single item labels
     * @param boolean $multiple     Enable multiple item selection
     *
     * @return Array   Configuration
     */
    protected function confSelect($description, $descriptions, $multiple = false)
    {
        // required properties
        return array(
            'type' => 'select',
            'description' => $description,
            'descriptions' => $descriptions,
            'multiple' => $multiple,
        );
    }

    /**
     * throws styled message
     *
     * @param string $type Type of message ('ERROR', 'SUCCESS')
     * @param string $text Content of message
     *
     * @return string HTML content
     */
    protected function throwMessage($text, $type)
    {
        return '<div class="'
                . strtolower(self::PLUGIN_TITLE . '-' . $type)
            . '">'
            . '<div>'
                . $this->_cms_lang->getLanguageValue(strtolower($type))
            . '</div>'
            . '<span>' . $text. '</span>'
            . '</div>';
    }

}

?>
